"""
Main.py
by Jason Conti
February 19, 2010

Applet for viewing recent notifications.
"""

import gnomeapplet
import gtk
import logging
import os.path

from string import Template

from About import show_about
from Icon import clear_icon_cache, load_icon
from MessageWindow import MessageWindow
from Notification import Notification
from Translations import _, ngettext

logger = logging.getLogger("Main")

class Main(object):
  """The main applet wrapper."""
  def __init__(self, applet, iid):
    self._applet = applet
    self._applet.set_background_widget(applet)
    self._applet.connect("change-size", self.on_change_size)

    self._unread_messages = 0

    self._applet_size = self.get_applet_size()

    icon_size = self.get_icon_size(self._applet_size)
    self.load_icons(icon_size)

    self._image = gtk.Image()
    self._image.set_from_pixbuf(self._read_icon)

    self._button = gtk.ToggleButton()
    self._button.set_relief(gtk.RELIEF_NONE)
    self._button.set_image(self._image)
    self._button.connect("button_press_event", self.do_not_eat_button_press)
    self._button.connect("toggled", self.on_button_toggled)
    self._applet.add(self._button)

    self._applet.setup_menu(Template("""
      <popup name="button3">
        <menuitem name="Clear" verb="Clear" _label="$clear_label" pixtype="stock" pixname="gtk-clear"/>
        <separator/>
        <menuitem name="About" verb="About" _label="$about_label" pixtype="stock" pixname="gtk-about"/>
      </popup>
      """).substitute(
          about_label=_("_About"), 
          clear_label=_("_Clear Messages")), [
      ("Clear", lambda a, b: self.on_clear_messages(a)),
      ("About", lambda a, b: self.on_show_about(a))
      ])

    self._window = MessageWindow(self)

    self._applet.connect("destroy", self.on_applet_destroyed)
    self._applet.show_all()

    self.update()

    self._notify = Notification()
    self._notify.connect("message-received", self.on_message_received)

  def get_applet_size(self):
    """Gets the size of the applet."""
    return self._applet.get_size()

  def get_icon_size(self, size):
    """Gets the icon size from the given size."""
    if size < 22:
      size = 16
    elif size < 24:
      size = 22
    elif size < 32:
      size = 24
    elif size < 48:
      size = 32
    else:
      size = 48

    return size

  def get_orient(self):
    """Gets the orientation of the panel."""
    return self._applet.get_orient()

  def get_origin(self):
    """Gets the position of the button window."""
    return self._button.get_window().get_origin()

  def get_screen(self):
    """Gets the screen the button window is on."""
    return self._button.get_window().get_screen()

  def get_size(self):
    """Gets the size of the button window."""
    return self._button.get_window().get_size()

  def load_icons(self, size):
    """Loads icons from the default icon theme."""
    self._read_icon = load_icon("humanity-notification-read", size)
    self._unread_icon = load_icon("humanity-notification-unread", size)

  def update(self):
    """Updates the applet icon and unread messages tooltip."""
    self.update_applet_image()
    self.update_unread_messages()

  def update_applet_image(self):
    """Update the applet icon if it has changed."""
    if self._unread_messages > 0:
      icon = self._unread_icon
    else:
      icon = self._read_icon

    if icon != self._image.get_pixbuf():
      self._image.set_from_pixbuf(icon)

  def update_unread_messages(self):
    """Updates the unread message tooltip."""
    n = self._unread_messages
    label = ngettext("1 unread message", "{0} unread messages", n).format(n)
    self._button.set_tooltip_text(label)

  # Callbacks

  def do_not_eat_button_press(self, widget, event):
    """Adapted from the GNOME clock applet."""
    if event.button != 1:
      widget.stop_emission("button_press_event")

    return False

  def on_change_size(self, applet, size):
    """Update the icon size if the panel size changes."""
    if self._applet_size == size:
      return

    self._applet_size = size
    icon_size = self.get_icon_size(size)
    self.load_icons(icon_size)
    self.update_applet_image()

  def on_applet_destroyed(self, *args):
    """Stop the message notifications before destroying the applet."""
    self._notify.close()

  def on_button_toggled(self, button, *args):
    """Show/Hide the message window when the button is toggled."""
    if button.get_active():
      self._window.show_all()
      self._window.align_with_parent()
    else:
      self._window.hide_all()
      self._window.clear_selections()
      self._unread_messages = 0
      self.update()

  def on_clear_messages(self, *args):
    """Clear messages from the message window."""
    self._window.clear_messages()
    clear_icon_cache()

  def on_message_received(self, monitor, message):
    """Received a message from the notifier."""
    # Discard volume notifications
    if not message.is_volume_notification():
      self._window.add_message(message)
      self._unread_messages += 1
      self.update()

  def on_show_about(self, *args):
    """Show the about dialog."""
    show_about(self)

